@@ -1,42 +1,49 @@
-"""
-Collector plugin for maven artifacts
-"""
+"""Collector plugin for maven artifacts."""
+
 from __future__ import annotations
 
+import importlib
 import os
 
 from pathlib import Path
-from subprocess import CalledProcessError
+from subprocess import CalledProcessError, CompletedProcess
 from tempfile import NamedTemporaryFile
-from typing import Any, OrderedDict
+from typing import TYPE_CHECKING, Any
 
 import jmespath
 import xmltodict
 
-from packageurl import PackageURL
+from pydantic import SecretStr
+
+import hoppr.net
+import hoppr.utils
 
 from hoppr import __version__
 from hoppr.base_plugins.collector import SerialCollectorPlugin
 from hoppr.base_plugins.hoppr import hoppr_rerunner
-from hoppr.models import HopprContext
-from hoppr.models.credentials import CredentialRequiredService
-from hoppr.models.manifest import Component
 from hoppr.result import Result
 
+if TYPE_CHECKING:
+    from collections import OrderedDict
+
+    from hoppr.models import HopprContext
+    from hoppr.models.credentials import CredentialRequiredService
+    from hoppr.models.sbom import Component
+
+
 _MAVEN_DEP_PLUGIN = "org.apache.maven.plugins:maven-dependency-plugin:3.5.0"
 
 
-class CollectMavenPlugin(SerialCollectorPlugin):
-    """
-    Collector plugin for maven artifacts
-    """
+class CollectMavenPlugin(
+    SerialCollectorPlugin,
+    required_commands=["mvn"],
+    supported_purl_types=["maven"],
+    products=["maven/*"],
+    system_repositories=["https://repo.maven.apache.org/maven2"],
+):
+    """Collector plugin for maven artifacts."""
 
-    supported_purl_types = ["maven"]
-    required_commands = ["mvn"]
-    products: list[str] = ["maven/*"]
-    system_repositories: list[str] = ["https://repo.maven.apache.org/maven2"]
-
-    def get_version(self) -> str:  # pylint: disable=duplicate-code
+    def get_version(self) -> str:  # noqa: D102
         return __version__
 
     def __init__(self, context: HopprContext, config: dict | None = None) -> None:
@@ -65,40 +72,35 @@ class CollectMavenPlugin(SerialCollectorPlugin):
                         expression="settings.profiles.profile[].repositories.repository[].url", data=settings_dict
                     )
 
-                    for repo in repo_urls:
+                    for repo in repo_urls or []:
                         if repo not in self.system_repositories:
                             self.system_repositories.append(repo)
 
-    def _get_maven_component(self, command: list[str], password_list: list[str], **kwargs):
+    def _get_maven_component(self, command: list[str], password_list: list[str], **kwargs) -> CompletedProcess[bytes]:
         full_command = command.copy()
-        for key, value in kwargs.items():
-            full_command.append(f"-D{key}={value}")
-
+        full_command.extend(f"-D{key}={value}" for key, value in kwargs.items())
         result = self.run_command(full_command, password_list)
 
         # The maven plugin does not recognize the 'destFileName' argument, so rename the file
         if result.returncode == 0:
             _, name, version, extension = kwargs.get("artifact", "::::").split(":")
-            directory = kwargs.get("outputDirectory", ".")
+            directory: Path = kwargs.get("outputDirectory", ".")
             old_name = f"{name}-{version}.{extension}"
             new_name = kwargs.get("destFileName", f"{name}_{version}.{extension}")
 
-            os.rename(directory / old_name, directory / new_name)
+            (directory / old_name).rename(directory / new_name)
 
         return result
 
     @hoppr_rerunner
     def collect(self, comp: Component, repo_url: str, creds: CredentialRequiredService | None = None) -> Result:
-        """
-        Copy a component to the local collection directory structure
-        """
-        # pylint: disable=too-many-locals
-
-        purl: PackageURL = PackageURL.from_string(comp.purl)  # pyright: ignore[reportGeneralTypeIssues]
+        """Copy a component to the local collection directory structure."""
+        purl = hoppr.utils.get_package_url(comp.purl)
         artifact = f"{purl.namespace}:{purl.name}:{purl.version}"
 
         extension = purl.qualifiers.get("type", "tar.gz")
-        target_dir = self.directory_for(purl.type, repo_url, subdir=f"{purl.namespace}")
+        target_dir = self.directory_for(purl.type, repo_url, subdir=purl.namespace)
+        package_out_path = target_dir / f"{purl.name}_{purl.version}.{extension}"
 
         self.get_logger().info(msg="Copying maven artifact:", indent_level=2)
         self.get_logger().info(msg=f"source: {repo_url}", indent_level=3)
@@ -129,7 +131,7 @@ class CollectMavenPlugin(SerialCollectorPlugin):
                 "remoteRepositories": f"repoId::::{repo_url}",
             }
 
-            if creds is not None:
+            if creds is not None and isinstance(creds.password, SecretStr):
                 defines["repo.login"] = creds.username
                 defines["repo.pwd"] = creds.password.get_secret_value()
                 password_list = [creds.password.get_secret_value()]
@@ -139,6 +141,7 @@ class CollectMavenPlugin(SerialCollectorPlugin):
                 f"{_MAVEN_DEP_PLUGIN}:copy",
                 f"--settings={settings_file.name}",
                 *self.extra_opts,
+                *(["--debug"] if self.get_logger().is_verbose() else []),
             ]
 
             run_result = self._get_maven_component(command, password_list, **defines)
@@ -158,6 +161,11 @@ class CollectMavenPlugin(SerialCollectorPlugin):
                 self.get_logger().debug(msg=error_msg, indent_level=2)
                 return Result.retry(message=error_msg)
 
-        self.set_collection_params(comp, repo_url, target_dir)
+        self.set_collection_params(comp, repo_url, package_out_path)
 
         return Result.success(return_obj=comp)
+
+
+if os.getenv("HOPPR_EXPERIMENTAL"):
+    module = importlib.import_module(name="hoppr.plugins.collect.maven")
+    CollectMavenPlugin = module.CollectMavenPlugin  # type: ignore[misc]
