@@ -1,43 +1,63 @@
-"""
-Collector plugin for raw files
-"""
+"""Collector plugin for raw files."""
+
+from __future__ import annotations
 
 import shutil
 
 from pathlib import Path
-from typing import Any, List
+from typing import TYPE_CHECKING
 from urllib.parse import unquote
 
-from packageurl import PackageURL
+import hoppr.utils
 
 from hoppr import __version__
 from hoppr.base_plugins.collector import SerialCollectorPlugin
 from hoppr.base_plugins.hoppr import hoppr_rerunner
-from hoppr.models.credentials import CredentialRequiredService
+from hoppr.exceptions import HopprPluginError
 from hoppr.models.types import RepositoryUrl
 from hoppr.net import download_file
 from hoppr.result import Result
 
+if TYPE_CHECKING:
+    from hoppr.models.credentials import CredentialRequiredService
+    from hoppr.models.sbom import Component
 
-class CollectRawPlugin(SerialCollectorPlugin):
-    """
-    Collector plugin for raw files
-    """
 
-    supported_purl_types = ["binary", "generic", "raw"]
-    products: List[str] = ["binary/*", "generic/*", "raw/*"]
+class CollectRawPlugin(
+    SerialCollectorPlugin,
+    supported_purl_types=["binary", "generic", "raw"],
+    products=["binary/*", "generic/*", "raw/*"],
+):
+    """Collector plugin for raw files."""
 
-    def get_version(self) -> str:
+    def get_version(self) -> str:  # noqa: D102
         return __version__
 
+    def _validate_hashes(self, comp: Component, file_location: Path):
+        """Validates the component hashes against hashes generated for each component.
+
+        If no conflicting hashes are found, update the component with the generated hashes.
+
+        Args:
+            comp: Component object whose hashes will be validated.
+            file_location: Path to the file to generate hashes from.
+
+        Raises:
+            HopprPluginError: If hash validation fails.
+        """
+        generated_hashes = self._get_artifact_hashes(file_location)
+
+        try:
+            comp.update_hashes(generated_hashes)
+        except ValueError as ex:
+            raise HopprPluginError(f"Hash for {comp.name} does not match expected hash.") from ex
+
     @hoppr_rerunner
-    def collect(self, comp: Any, repo_url: str, creds: CredentialRequiredService | None = None):
-        """
-        Copy a component to the local collection directory structure
-        """
+    def collect(self, comp: Component, repo_url: str, creds: CredentialRequiredService | None = None) -> Result:
+        """Copy a component to the local collection directory structure."""
         source_url = RepositoryUrl(url=repo_url)
 
-        purl = PackageURL.from_string(comp.purl)
+        purl = hoppr.utils.get_package_url(comp.purl)
 
         subdir = None
         if purl.namespace is not None:
@@ -63,7 +83,9 @@ class CollectRawPlugin(SerialCollectorPlugin):
 
             shutil.copy(source_file, target_dir)
 
-            self.set_collection_params(comp, repo_url, target_dir)
+            self._validate_hashes(comp, target_dir.joinpath(file_name))
+            self.set_collection_params(comp, repo_url, target_dir / file_name)
+
             return Result.success(return_obj=comp)
 
         download_url = source_url / file_name
@@ -72,7 +94,7 @@ class CollectRawPlugin(SerialCollectorPlugin):
         self.get_logger().info(msg=f"source: {download_url}", indent_level=3)
         self.get_logger().info(msg=f"destination: {target_dir.joinpath(file_name)}", indent_level=3)
 
-        response = download_file(f"{download_url}", str(target_dir / file_name), creds)
+        response = download_file(f"{download_url}", str(target_dir / file_name), creds, timeout=self.process_timeout)
         result = Result.from_http_response(response)
 
         if result.is_fail():
@@ -82,6 +104,6 @@ class CollectRawPlugin(SerialCollectorPlugin):
         if not result.is_success():
             return result
 
-        self.set_collection_params(comp, repo_url, target_dir)
+        self.set_collection_params(comp, repo_url, target_dir / file_name)
 
         return Result.success(return_obj=comp)
