@@ -1,21 +1,37 @@
-"""
-Collector plugin for Yum packages
-"""
+"""Collector plugin for Yum packages."""
+
 from __future__ import annotations
 
-from pathlib import Path
-from typing import Any
+import warnings
 
-from packageurl import PackageURL
+from pathlib import Path
+from typing import TYPE_CHECKING
+
+import hoppr.utils
 
 from hoppr import __version__
 from hoppr.base_plugins.collector import SerialCollectorPlugin
 from hoppr.base_plugins.hoppr import hoppr_rerunner
-from hoppr.models import HopprContext
+from hoppr.models.credentials import CredentialRequiredService
 from hoppr.result import Result
 
+if TYPE_CHECKING:
+    from packageurl import PackageURL
 
-def _artifact_string(purl):
+    from hoppr.models import HopprContext
+    from hoppr.models.sbom import Component
+
+
+warnings.filterwarnings(action="once", category=DeprecationWarning)
+
+warnings.warn(
+    message="The YUM collector plugin has been deprecated; use either the DNF or RPM collector instead",
+    category=DeprecationWarning,
+    stacklevel=1,
+)
+
+
+def _artifact_string(purl: PackageURL) -> str:
     artifact_string = purl.name
     if purl.version is not None:
         artifact_string += f"-{purl.version}"
@@ -28,31 +44,25 @@ def _artifact_string(purl):
     return artifact_string
 
 
-class CollectYumPlugin(SerialCollectorPlugin):
-    """
-    Collector plugin for yum images
-    """
+class CollectYumPlugin(
+    SerialCollectorPlugin,
+    supported_purl_types=["rpm"],
+    required_commands=["yumdownloader"],
+    products=["rpm/*"],
+):
+    """Collector plugin for yum images."""
 
-    supported_purl_types = ["rpm"]
-    required_commands = ["yumdownloader"]
-    products: list[str] = ["rpm/*"]
-
-    def get_version(self) -> str:
+    def get_version(self) -> str:  # noqa: D102
         return __version__
 
     def __init__(self, context: HopprContext, config: dict | None = None) -> None:
         super().__init__(context=context, config=config)
-        if self.config is not None:
-            if "yumdownloader_command" in self.config:
-                self.required_commands = [self.config["yumdownloader_command"]]
+        self.required_commands = (self.config or {}).get("yumdownloader_command", self.required_commands)
 
     @hoppr_rerunner
-    def collect(self, comp: Any, repo_url: str, creds=None):
-        """
-        Copy a component to the local collection directory structure
-        """
-
-        purl: PackageURL = PackageURL.from_string(comp.purl)  # pyright: ignore[reportGeneralTypeIssues]
+    def collect(self, comp: Component, repo_url: str, creds: CredentialRequiredService | None = None) -> Result:
+        """Copy a component to the local collection directory structure."""
+        purl = hoppr.utils.get_package_url(comp.purl)
         artifact = _artifact_string(purl)
 
         self.get_logger().info(msg=f"Copying yum package from {comp.purl}", indent_level=2)
@@ -67,6 +77,9 @@ class CollectYumPlugin(SerialCollectorPlugin):
             artifact,
         ]
 
+        if self.get_logger().is_verbose():
+            command = [*command, "--verbose"]
+
         run_result = self.run_command(command)
         if run_result.returncode != 0:
             msg = (
